/**
 * @file aisl/server.h
 *
 * Copyright (c) 2017-2019 by Löwenware Ltd.
 *
 * Project homepage: https://lowenware.com/aisl/
 *
 */

#ifndef AISL_SERVER_H_CC564608_7A05_4B31_9E7E_32750BC60768
#define AISL_SERVER_H_CC564608_7A05_4B31_9E7E_32750BC60768

#include <arpa/inet.h>
#include <aisl/types.h>

/**
 * @brief Function to get appropriate AISL instance pointer from server pointer.
 * @param server an #aisl_server_t pointer.
 * @return an #aisl_t instance pointer.
 */
aisl_t
aisl_server_get_instance( aisl_server_t server );


/**
 * @brief Copies server listen address information to sockaddr_in structure.
 * @param server an #aisl_server_t pointer.
 * @param address a pointer to sockaddr_in structure.
 */
void
aisl_server_get_address( aisl_server_t server, struct sockaddr_in * address);


#ifndef AISL_WITHOUT_SSL

/**
 * @brief Function to switch on and off SSL for the #aisl_server_t.
 * @param server an #aisl_server_t pointer.
 * @param value a boolean value representing SSL enabled/disabled state.
 */
void
aisl_server_set_ssl( aisl_server_t server, bool value );


/**
 * @brief Function to get on and off status of SSL for the #aisl_server_t.
 * @param server an #aisl_server_t pointer.
 * @return a boolean value representing SSL enabled/disabled state.
 */
bool
aisl_server_get_ssl( aisl_server_t server );

#endif

#endif /* !AISL_SERVER_H */
