#include <stdlib.h>
#include <stdio.h>
#include <stdarg.h>
#include <string.h>
#include "buffer.h"


int32_t
buffer_init( buffer_t buffer, int32_t size )
{
  if ((buffer->data = malloc(size)) != NULL)
  {
    buffer->size = size;
    buffer->length = 0;
    return size;
  }

  return -1;
}


void
buffer_release( buffer_t buffer )
{
  if (buffer->data)
    free(buffer->data);

  buffer->length = 0;
  buffer->size = 0;
}


int32_t
buffer_add( buffer_t self, const char * data, int32_t size )
{
  int32_t result = size+self->size;

  char * ptr;

  if ( (ptr = realloc(self->data, result+1)) != NULL )
  {
    memcpy( &ptr[self->size], data, size );
    ptr[ result ] = 0;
    self->data = ptr;
    self->size = result;
  }
  else
    result = -1;

  return result;
}


int32_t
buffer_clear( buffer_t self, int32_t to_alloc )
{
  self->length = 0;

  if (to_alloc > 0)
  {
    if ( (self->data = realloc(self->data, to_alloc)) != NULL )
    {
      self->size = to_alloc;
      return to_alloc;
    }
    else
      to_alloc = -1;
  }

  free(self->data);
  self->data = NULL;

  return to_alloc;
}


int32_t
buffer_shift( buffer_t self, int32_t size )
{
  int32_t result;

  if (size && !(size > self->size))
  {
    result = self->size - size;
    memmove(self->data, &self->data[size], result);
    self->size = result;
  }
  else
    result = -1;

  return result;
}


int32_t
buffer_insert( buffer_t self, int32_t offset, const char * data, int32_t size )
{
  int32_t result = size + self->size;

  char * ptr;

  if ( (ptr = realloc(self->data, result+1)) != NULL )
  {
    memmove( &ptr[offset+size], &ptr[offset], self->size - offset );
    memcpy( &ptr[offset], data, size );
    ptr[ result ] = 0;
    self->data = ptr;
    self->size = result;
  }
  else
    result = -1;

  return result;
}
